/*******************************************************************************
 *
 * blueMarine - open source photo workflow
 * =======================================
 *
 * Copyright (C) 2003-2009 by Fabrizio Giudici
 * Project home page: http://bluemarine.tidalwave.it
 *
 *******************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0 
 * 
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License. 
 *
 *******************************************************************************
 *
 * $Id: BasicPersistenceSourceSink.java,v b89b0b568aeb 2010/02/17 15:26:52 fabrizio $
 *
 ******************************************************************************/
package it.tidalwave.metadata.basic.persistence;

import javax.annotation.Nonnull;
import java.util.Collection;
import java.util.Collections;
import java.util.Arrays;
import org.openide.loaders.DataObject;
import it.tidalwave.util.logging.Logger;
import it.tidalwave.metadata.Metadata;
import it.tidalwave.metadata.Metadata.FindOption;
import it.tidalwave.metadata.Metadata.StoreOption;
import it.tidalwave.metadata.MetadataItemHolder;
import it.tidalwave.metadata.spi.MetadataItemEnhancer;
import it.tidalwave.metadata.spi.MetadataItemSourceSinkSupport;
import it.tidalwave.metadata.basic.Basic;
import it.tidalwave.metadata.basic.BasicHolder;
import it.tidalwave.metadata.persistence.spi.BeanAccessor;
import it.tidalwave.metadata.persistence.spi.MetadataPersistenceSpi;
import it.tidalwave.metadata.persistence.spi.PropertyPersistenceBeanAccessor;

/*******************************************************************************
 *
 * This class provides persistence support for {@link Basic} metadata items.
 * 
 * @author  Fabrizio Giudici
 * @version $Id: BasicPersistenceSourceSink.java,v b89b0b568aeb 2010/02/17 15:26:52 fabrizio $
 *
 ******************************************************************************/
public class BasicPersistenceSourceSink extends MetadataItemSourceSinkSupport<Basic>
  {
    private final static String CLASS = BasicPersistenceSourceSink.class.getName();
    private final static Logger logger = Logger.getLogger(CLASS);
    
    protected final BeanAccessor BEAN_ACCESSOR =
        new PropertyPersistenceBeanAccessor(Basic.class, "Basic", BasicPersistenceSourceSink.class, "BasicCodes.properties");

    @Nonnull
    private final MetadataPersistenceSpi persistenceSpi = MetadataPersistenceSpi.Locator.findPersistenceSpi();
      
    /***************************************************************************
     *
     *
     **************************************************************************/
    public BasicPersistenceSourceSink()
      throws Exception
      {
        super(Basic.class, "Basic Persistence", Metadata.StorageType.INTERNAL);
        persistenceSpi.register(Basic.class, BEAN_ACCESSOR);
      }
    
    /***************************************************************************
     *
     * {@inheritDocs}
     *
     **************************************************************************/
    @Nonnull
    public Collection<? extends MetadataItemHolder<Basic>> loadMetadataItems (@Nonnull final DataObject dataObject,
                                                                              @Nonnull final FindOption ... options)
      throws Exception
      {
        logger.fine("loadMetadataItem(%s, %s)", dataObject, Arrays.toString(options));

        final Basic basic = MetadataItemEnhancer.createEnhancedItem(new Basic());
        final BasicHolder holder = new BasicHolder(basic);
        persistenceSpi.refresh(dataObject, holder);
        
        return Collections.singletonList(holder);
      }

    /***************************************************************************
     *
     * {@inheritDocs}
     *
     **************************************************************************/
    public void storeMetadataItem (@Nonnull final DataObject dataObject,
                                   @Nonnull final MetadataItemHolder<Basic> holder,
                                   @Nonnull final StoreOption... options)
      throws Exception
      {
        logger.fine("storeMetadataItem(%s, %s, %s)", dataObject, holder, Arrays.toString(options));
        persistenceSpi.manage(dataObject, holder, options);
      }
  }
